#ifndef EMBEDDING_H
#define EMBEDDING_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _EmbeddingResult EmbeddingResult;

/// @brief 获取向量化结果
/// @param result 传入 向量化结果的结构体
/// @return float 数组 存储的向量数据
float *embedding_result_get_vector_data(EmbeddingResult *result);

/// @brief 获取向量化结果
/// @param result 传入 向量化结果的结构体
/// @return float 数组 数组长度
int embedding_result_get_vector_length(EmbeddingResult *result);

/// @brief 获取向量化错误码
/// @param result 传入 向量化结果的结构体
/// @return 错误码
int embedding_result_get_error_code(EmbeddingResult *result);

/// @brief 获取向量化错误信息
/// @param result 传入 向量化结果的结构体
/// @return 错误信息
const char *embedding_result_get_error_message(EmbeddingResult *result);

/// @brief  销毁向量化结果
/// @param result
void embedding_result_destroy(EmbeddingResult **result);

typedef struct _TextEmbeddingSession TextEmbeddingSession;

typedef void (*TextEmbeddingResultCallback)(EmbeddingResult *result,
                                            void *callback_user_data);

TextEmbeddingSession *embedding_te_create_session();
void embedding_te_destroy_session(TextEmbeddingSession **session);
int embedding_te_init_session(TextEmbeddingSession *session);

/// @brief 获取文本向量化模型名称
/// @param session 传入 embedding_te_create_session() 创建的 session
/// @return 返回模型名称
const char *embedding_te_get_current_model_name(TextEmbeddingSession *session);

/// @brief 获取文本向量化模型版本
/// @param session 传入 embedding_te_create_session() 创建的 session
/// @return 返回模型版本
const char *embedding_te_get_current_model_version(
    TextEmbeddingSession *session);

/// @brief 文本向量化模型 同步接口 向量化文本
/// @param session 传入 embedding_te_create_session() 创建的 session
/// @param text 传入 待向量化的文本
/// @param is_question 传入 是否开启提示词
/// @return 返回向量化结果
EmbeddingResult *embedding_te_embedding_text(TextEmbeddingSession *session,
                                             const char *text,
                                             bool is_question);

/// @brief 文本向量化模型 设置异步并发接口的向量化结果回调函数。
/// @param session 传入 embedding_te_create_session() 创建的 session
/// @param callback 传入 TextEmbeddingResultCallback 类型的结果回调函数
/// @param callback_user_data 调用者自定义的数据，作为 callback 的参数返回 用来区分 callback
void embedding_te_set_result_callback(TextEmbeddingSession *session,
                                      TextEmbeddingResultCallback callback,
                                      void *callback_user_data);

/// @brief 文本向量化模型 异步并发接口 向量化文本
/// @param session 传入 embedding_te_create_session() 创建的 session
/// @param text 传入 待向量化的文本
/// @param is_question 传入 是否开启提示词
void embedding_te_embedding_text_with_callback_async(
    TextEmbeddingSession *session, const char *text, bool is_question);

/// @brief 文本向量化模型 异步不并发接口 向量化文本
/// @param session 传入 embedding_te_create_session() 创建的 session
/// @param text 传入 待向量化的文本
/// @param is_question 传入 是否开启提示词
/// @param callback 向量化结果回调函数 传入 TextEmbeddingResultCallback 类型的结果回调函数
/// @param callback_user_data 调用者自定义的数据，作为 callback 的参数返回
void embedding_te_embedding_text_async(TextEmbeddingSession *session,
                                       const char *text, bool is_question,
                                       TextEmbeddingResultCallback callback,
                                       void *callback_user_data);

typedef struct _ImageEmbeddingSession ImageEmbeddingSession;

typedef void (*ImageEmbeddingResultCallback)(EmbeddingResult *result,
                                             void *callback_user_data);

ImageEmbeddingSession *embedding_im_create_session();
void embedding_im_destroy_session(ImageEmbeddingSession **session);
int embedding_im_init_session(ImageEmbeddingSession *session);

/// @brief 获取图像向量化模型名称
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @return 返回模型名称
const char *embedding_im_get_current_model_name(ImageEmbeddingSession *session);

/// @brief 获取图像向量化模型版本
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @return 返回模型版本
const char *embedding_im_get_current_model_version(
    ImageEmbeddingSession *session);

/// @brief 图像向量化模型同步接口 向量化文本
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @param text 传入 待向量化的文本
/// @param is_question 传入 是否开启提示词
/// @return 返回向量化结果
EmbeddingResult *embedding_im_embedding_text(ImageEmbeddingSession *session,
                                             const char *text,
                                             bool is_question);

/// @brief 图像向量化模型同步接口 向量化图像
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @param image_file 传入 待向量化的图像路径
/// @return 返回向量化结果
EmbeddingResult *embedding_im_embedding_image_file(
    ImageEmbeddingSession *session, const char *image_file);

/// @brief 图像向量化模型同步接口 向量化图像
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @param image_data 传入 待向量化的base64图像数据
/// @param image_data_length 传入 base64图像数据的长度
/// @return 返回向量化结果
EmbeddingResult *embedding_im_embedding_base64_image_data(
    ImageEmbeddingSession *session, const unsigned char *image_data,
    unsigned int image_data_length);

/// @brief 图像向量化模型 设置异步并发接口的向量化结果回调函数。
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @param callback 传入 ImageEmbeddingResultCallback 类型的结果回调函数
/// @param callback_user_data 调用者自定义的数据，作为 callback 的参数返回 用来区分 callback
void embedding_im_set_result_callback(ImageEmbeddingSession *session,
                                      ImageEmbeddingResultCallback callback,
                                      void *callback_user_data);

/// @brief 图像向量化模型 异步并发接口 向量化文本
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @param is_question 传入 是否开启提示词
/// @param text 传入 待向量化的文本
void embedding_im_embedding_text_with_callback_async(
    ImageEmbeddingSession *session, const char *text, bool is_question);

/// @brief 图像向量化模型 异步并发接口 向量化图像
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @param image_file 传入 待向量化的图像路径
void embedding_im_embedding_image_file_with_callback_async(
    ImageEmbeddingSession *session, const char *image_file);

/// @brief 图像向量化模型 异步并发接口 向量化图像
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @param image_data 传入 待向量化的base64图像数据
/// @param image_data_length 传入 base64图像数据的长度
void embedding_im_embedding_base64_image_data_with_callback_async(
    ImageEmbeddingSession *session, const unsigned char *image_data,
    unsigned int image_data_length);

/// @brief 图像向量化模型 异步不并发接口 向量化文本
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @param text 传入 待向量化的文本
/// @param is_question 传入 是否开启提示词
/// @param callback 向量化结果回调函数 传入 ImageEmbeddingResultCallback 类型的结果回调函数
/// @param callback_user_data 调用者自定义的数据，作为 callback 的参数返回
void embedding_im_embedding_text_async(ImageEmbeddingSession *session,
                                       const char *text, bool is_question,
                                       ImageEmbeddingResultCallback callback,
                                       void *callback_user_data);

/// @brief 图像向量化模型 异步不并发接口 向量化图像
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @param file_path 传入 待向量化的图像路径
/// @param callback 向量化结果回调函数 传入 ImageEmbeddingResultCallback 类型的结果回调函数
/// @param callback_user_data 调用者自定义的数据，作为 callback 的参数返回
void embedding_im_embedding_image_file_async(
    ImageEmbeddingSession *session, const char *file_path,
    ImageEmbeddingResultCallback callback, void *callback_user_data);

/// @brief 图像向量化模型 异步不并发接口 向量化图像
/// @param session 传入 embedding_im_create_session() 创建的 session
/// @param image_data 传入 待向量化的base64图像数据
/// @param image_data_length 传入 base64图像数据的长度
/// @param callback 向量化结果回调函数 传入 ImageEmbeddingResultCallback 类型的结果回调函数
/// @param callback_user_data 调用者自定义的数据，作为 callback 的参数返回
void embedding_im_embedding_base64_image_data_async(
    ImageEmbeddingSession *session, const unsigned char *image_data,
    unsigned int image_data_length, ImageEmbeddingResultCallback callback,
    void *callback_user_data);

#ifdef __cplusplus
}
#endif

#endif
